; Definitions and macros used by the testing ROM.

   define   LED_PORT 63
   define   ULA_PORT 254
   define   ROMPAGE_PORT   31
   define   BORDERBLU 1
   define   BORDERRED 2
   define   LM_FLASH #f0   ; rightmost 4 lights
   define   HM_FLASH #0f   ; leftmost 4 lights
   define   OK_FLASH #ff   ; all lights
   define   ERR_FLASH   #aa   ; alternate lights

   MACRO PAUSE
      exx
      ld bc, #ffff
.pauseloop
      nop
      nop
      nop
      nop
      nop
      djnz .pauseloop
      dec b    ; wrap b
      dec c
      jr nz, .pauseloop
      exx
   ENDM

; Flash the value in A four times
   MACRO FLASH
      ld b, 4
.flashloop      
      out (LED_PORT), a
      PAUSE
      ex af, af'
      ld a, 0
      out (LED_PORT), a
      ex af, af'
      PAUSE
      djnz .flashloop
   ENDM      

   MACRO WALKLOOP start, length
      ld hl, start
      ld de, length
.walk.loop
      ld a, 1
      ld b, 8     ; test 8 bits
      or a        ; ensure carry is cleared
.walk.checkbits
      ld (hl), a
      ld c, a     ; for compare
      ld a, (hl)
      cp c
      jr nz, .walk.borked
      rla
      djnz .walk.checkbits
      inc hl
      dec de
      ld a, d     ; does de=0?
      or e
      jr z, .walk.done
      jr .walk.loop
.walk.borked
      BORKEDFLASH start
.walk.done
   ENDM
 
   MACRO BORKEDFLASH location
      ld ix, location
.borked
      ld a, BORDERRED
      out (ULA_PORT), a
      ld a, ERR_FLASH
      FLASH
      ld b, 8
.borked.loop
      xor a
      ld (hl), a
      PAUSE
      ld a, (hl)
      out (LED_PORT), a
      PAUSE
      ld a, 255
      ld (hl), a
      PAUSE
      ld a, (hl)
      out (LED_PORT), a
      djnz .borked.loop
   ENDM

   MACRO BLANKMEM pattern
      ld hl,16384
.blankloop
      ld (hl), pattern
      inc hl
      ld a, h
      or l
      jr nz, .blankloop
   ENDM

   MACRO ALTPATA start,len,fill
	BLANKMEM fill
	ld hl, start
	ld bc, len
.altpat1.wrloop1
	ld a, fill
	cpl
	ld (hl), a
	inc hl
	inc hl
	dec bc
	dec bc
	ld a, b
	or c
	jr nz, .altpat1.wrloop1
.altpat1.rd
	ld hl, start
	ld bc, len
.altpat1.rdloop1
	ld a, fill
	cpl
	cp (hl)
	jr nz, .altpat1.borked
	inc hl
	dec bc
	cpl
	cp (hl)
	jr nz, .altpat1.borked
	inc hl
	dec bc
	ld a, b
	or c
	jr nz, .altpat1.rdloop1 
	jr .altpat1.done
.altpat1.borked
	ALTBORKED start
.altpat1.done
   ENDM

   MACRO ALTPATB start,len,fill
	BLANKMEM fill
	ld hl, start
	ld bc, len
.altpat2.wrloop1
	ld a, fill
	cpl
	inc hl
	ld (hl), a
	inc hl
	dec bc
	dec bc
	ld a, b
	or c
	jr nz, .altpat2.wrloop1
.altpat2.rd
	ld hl, start
	ld bc, len
.altpat2.rdloop1
	ld a, fill
	cp (hl)
	jr nz, .altpat2.borked
	inc hl
	dec bc
	cpl
	cp (hl)
	jr nz, .altpat2.borked
	inc hl
	dec bc
	ld a, b
	or c
	jr nz, .altpat2.rdloop1 
	jr .altpat2.done
.altpat2.borked
	ALTBORKED start
.altpat2.done
   ENDM	

   MACRO ALTBORKED location
      ld ix, location
      ld e, a		; save expected pattern
      ld a, BORDERRED
      out (ULA_PORT), a
      ld a, ERR_FLASH
      FLASH
      ld b, 4
.borked.loop
      ld a, e		; test pattern that failed
      out (LED_PORT), a
      PAUSE 
      ld a, (hl)
      out (LED_PORT), a
      PAUSE
      djnz .borked.loop
   ENDM

; see http://map.tni.nl/sources/external/z80bits.html#3.2 for the
; basis of the random fill
; IX = seed
   MACRO RAND16
      ld d, b
      ld e, c
      ld a, d
      ld h, e
      ld l, 253
      or a
      sbc hl, de
      sbc a, 0
      sbc hl, de
      ld d, 0
      sbc a, d
      ld e, a
      sbc hl, de
      jr nc, .rand16.done
      inc hl
.rand16.done
      ld b, h
      ld c, l
   ENDM

; note we use the stack to push the 16-bit random numbers in so addr
; should equal the top of the memory bank and reps should be half the
; size in bytes
   MACRO RANDFILL addr, reps
      ld sp, addr
      exx
      ld bc, 0    ; seed
      exx
      ld bc, reps
.randfill.loop      
      exx
      RAND16
      push hl
      exx
      dec bc
      ld a, b
      or c
      jr nz, .randfill.loop

.randfill.test      
      ld hl, addr
      exx
      ld bc, 0       ; seed
      exx
      ld bc, reps
.randfill.testloop
      dec hl
      dec hl
      ld sp, hl
      exx
      RAND16	; byte pair to test now in HL
      pop de	; get corresponding pair of bytes from RAM
      ld a, h
      cp d
      jr nz, .randfill.borked1
      ld a, l
      cp e
      jr nz, .randfill.borked2
      exx
      dec bc
      ld a, b
      or c
      jr nz, .randfill.testloop
      jr .randfill.done
.randfill.borked1
      ld e, d	; bad byte should be in E
.randfill.borked2
      ld ix, addr-1
      ld c, a   ; save good byte
      ld a, BORDERRED
      out (ULA_PORT), a
      ld b, 4
.randfill.borkedloop
      ld a, c   ; restore good byte for display
      out (LED_PORT), a
      PAUSE
      ld a, e   ; get bad byte for display
      out (LED_PORT), a
      PAUSE
      djnz .randfill.borkedloop
.randfill.done
   ENDM
