      include "defines.asm"

      org 0
      ld ix, 0    ; using ix to store a flag to tell us whether upper
                  ; ram is good (if it is we continue testing)

      ; border blue
      ld a, BORDERBLU
      out (ULA_PORT), a

      ; set all RAM to zero.
      BLANKMEM 0

lowerram_walk      
      ; start lower RAM 'walking bit' test
      ld a, 1
      FLASH
      WALKLOOP 16384,16384

upperram_walk
      ld a, 2
      FLASH
      WALKLOOP 32768,32768

lowerram_inversion
      ld a, 4
      FLASH
      ALTPATA 16384, 16384, 0 
      ALTPATA 16384, 16384, 255
      ALTPATB 16384, 16384, 0
      ALTPATB 16384, 16384, 255


upperram_inversion
      ld a, 8
      FLASH
      ALTPATA 32768, 32768, 0
      ALTPATA 32768, 32768, 255
      ALTPATB 32768, 32768, 0
      ALTPATB 32768, 32768, 255

lowerram_random
      ld a, 16
      FLASH
      RANDFILL 32768, 8192

upperram_random
      ld a, 32
      FLASH
      RANDFILL 0, 16384

memdone      
      ; this gives the opportunity to visually see what's happening in
      ; lower memory in case there is a problem with it.
      BLANKMEM 0
setattrs      
      ld hl,22528
      ld bc, 768
setattrs.loop
      ld (hl), 6
      inc hl
      dec bc
      ld a, b
      or c
      jr nz, setattrs.loop

      ld b, ixh
      bit 7, b
      jr z, use_uppermem

      ; upper memory is no good, give up now.
      halt

use_uppermem
      ; initialize print routine
      xor a
      ld (v_column), a
      ld hl, #4000
      ld (v_row), hl

      ; init stack
      ld sp, #8100

      ld hl, str_umok
      call print
      ld hl, str_upperram
      call print
      ld hl, str_ok
      call print
      ld hl, str_lowerram
      call print
      bit 6, b
      jr z, print_lowerok
      ld hl, str_bad
      call print
      jr rom_test
      ld hl, str_ok
      call print
print_lowerok
      ld hl, str_ok
      call print

rom_test      
      ld hl, str_romcrc
      call print

      ; copy CRC routine to RAM
      ld hl, romcrc
      ld de, do_romcrc
      ld bc, endromcrc-romcrc
      ldir

      ; call CRC generator in RAM, CRC ends up in HL
      call do_romcrc
      ld de, v_hexstr
      call Num2Hex
      xor a
      ld (v_hexstr+4), a
      ld hl, v_hexstr
      call print

      ; test ULA's generation of interrupts
      ld hl, str_intr
      call print
      ld hl, 0
      ld (v_intcount), hl
      ld a, intvec / 256
      ld i, a
      im 2
      ei
intloop      
      halt     ; we'll start again as soon as an interrupt is raised
      ld a, (v_intcount)   ; low order byte of counter word
      out (LED_PORT), a    ; flash LEDs 
      cp 0
      jr nz, intloop
      ld hl, (v_intcount)
      ld a, h
      cp 6
      jr z, intloop.done
      ld de, v_hexstr
      call Num2Hex
      ld hl, v_hexstr
      call print
      ld a, '.'
      call putc_5by8
      ld a, '.'
      call putc_5by8
      jr intloop
intloop.done
      ld hl, str_complete
      call print
      di
      ld hl, 0
      ld (v_intcount), hl
      ei
waitloop      
      halt
      ld a, (v_intcount+1)
      cp 2
      jr nz, waitloop
      di 
      ld hl, str_pagein
      call print
      ld hl, pagein
      ld de, do_pagein
      ld bc, end_pagein-do_pagein
      ldir
      jp do_pagein

pagein
      ld a, %00100000   ; bit 5 = release /ROMCS
      out (ROMPAGE_PORT), a
      jp 0
end_pagein

      ; this routine prints a null terminated string pointed to by hl
print
      ld a, (hl)
      cp 0
      ret z
      call putc_5by8
      inc hl
      jr print

      BLOCK #0B0B-$,#FF
intservice
      push hl
      ld hl, (v_intcount)
      inc hl
      ld (v_intcount), hl
      pop hl
      ei
      reti
     
      include "crc16.asm" 
      include "print5by8.asm"

str_umok
      defb "48K Sinclair ZX Spectrum Diagnostics v0.1\nby Dylan 'Winston' Smith\n\n", 0
str_upperram 
      defb "Upper RAM: ", 0
str_lowerram 
      defb "Lower RAM: ", 0
str_ok defb "OK\n", 0
str_bad defb "Bad!\n", 0
str_romcrc defb "\nROM CRC (should be FD5E) is: ", 0
str_intr defb "\n\nTesting interrupts. (LEDs should flash)\n", 0
str_complete defb "\n\nTests complete.\n\nWill attempt to page in Spectrum ROM in\napproximately 10 seconds...\n\n", 0
str_pagein
      defb "Here goes", 0
      BLOCK #3A00-$,#FF
intvec      
      BLOCK #3C00-$,#0B
      
      include "charset.asm"
      BLOCK #4000-$,#FF
      org 32768
      include "vars.asm"
